// src/components/SmartPages.ts
import { defineComponent, h, onMounted, watch } from "vue";
import { debounce } from "ts-debounce";
import { injectCSS } from "@renovamen/dynamic-css";

// src/utils/breakPage.ts
var NEW_PAGE = "md-it-newpage";
var removeElements = (parent, selector) => {
  const elements = Array.from(parent.querySelectorAll(selector));
  for (const e of elements)
    parent.removeChild(e);
};
var breakPage = (id, height, top, bottom, left, right) => {
  const page = document.querySelector(`#${id}`);
  if (!page)
    return;
  removeElements(page, ".vue-smart-page-break");
  const contentH = height - top - bottom;
  const getPageBreakElement = (marginTop) => {
    const pageBreak = document.createElement("div");
    pageBreak.className = "vue-smart-page-break";
    pageBreak.style.marginTop = `${marginTop}px`;
    pageBreak.style.paddingBottom = `${bottom}px`;
    pageBreak.style.marginLeft = `-${left}px`;
    pageBreak.style.marginRight = `-${right}px`;
    return pageBreak;
  };
  let pageH = 0;
  const newPage = document.createElement("div");
  for (const child of Array.from(page.children)) {
    const style = window.getComputedStyle(child, null);
    const childH = child.clientHeight + parseInt(style.marginTop) + parseInt(style.marginBottom);
    if (pageH + childH > contentH || child.className === NEW_PAGE) {
      newPage.appendChild(getPageBreakElement(height - pageH - top));
      pageH = 0;
    }
    newPage.appendChild(child.cloneNode(true));
    pageH += childH;
  }
  page.innerHTML = newPage.innerHTML;
  page.style.paddingBottom = `${height - pageH - top}px`;
};

// src/components/SmartPages.ts
var SmartPages_default = defineComponent({
  name: "SmartPages",
  props: {
    id: {
      type: String,
      required: true
    },
    content: {
      type: String,
      required: false,
      default: ""
    },
    height: {
      type: Number,
      required: true
    },
    width: {
      type: Number,
      required: true
    },
    top: {
      type: Number,
      required: false,
      default: 0
    },
    bottom: {
      type: Number,
      required: false,
      default: 0
    },
    left: {
      type: Number,
      required: false,
      default: 0
    },
    right: {
      type: Number,
      required: false,
      default: 0
    },
    watch: {
      type: Array,
      required: false,
      default: () => []
    },
    watchDelay: {
      type: Array,
      required: false,
      default: () => []
    },
    beforeBreakPage: {
      type: Function,
      required: false,
      default: () => void 0
    },
    afterBreakPage: {
      type: Function,
      required: false,
      default: () => void 0
    }
  },
  setup(props, { expose }) {
    const id = `vue-smart-pages-${props.id}`;
    const updateCSS = () => injectCSS(
      `#${id} {
          padding: ${props.top}px ${props.right}px ${props.bottom}px ${props.left}px;
          width: ${props.width}mm;
        }`,
      id
    );
    const resolvePages = (delay) => {
      updateCSS();
      const resolveBreak = () => {
        breakPage(id, props.height, props.top, props.bottom, props.left, props.right);
        if (props.afterBreakPage)
          props.afterBreakPage();
      };
      if (props.beforeBreakPage) {
        const fn = props.beforeBreakPage();
        if (fn && typeof fn.then === "function") {
          if (delay)
            fn.then(() => setTimeout(resolveBreak, delay));
          else
            fn.then(resolveBreak);
          return;
        }
      }
      if (delay)
        setTimeout(resolveBreak, delay);
      else
        resolveBreak();
    };
    onMounted(() => {
      watch(
        () => [
          props.top,
          props.bottom,
          props.left,
          props.right,
          props.width,
          props.content,
          props.height,
          ...props.watch
        ],
        () => debounce(resolvePages, 200)()
      );
      watch(
        () => props.watchDelay,
        () => debounce(() => resolvePages(100), 200)()
      );
      updateCSS();
    });
    expose({
      resolvePages
    });
    return () => h("div", {
      class: "vue-smart-pages",
      id,
      innerHTML: props.content
    });
  }
});

// src/index.ts
var src_default = SmartPages_default;
export {
  src_default as default
};
